#rem

Software to control Head's head through programming cable.
Demonstrates controlling a robot over serial and network connection (with RSBServer and RSBClient).
Ripped from Head's original software.

$Id: head-control.bas 336 2010-09-09 17:28:20Z nuumio $

#endrem

' Constants for output pins
symbol PIN_SOUND        = 0 ' Piezo speaker
symbol PIN_SERVO        = 1 ' Servo (and SRF05 trigger)
symbol PIN_LED_LR       = 2 ' LED: Left Red
symbol PIN_LED_LG       = 3 ' LED: Left Green
symbol PIN_MRF          = 4 ' Right motor forward
symbol PIN_MRB          = 5 ' Right motor backward
symbol PIN_MLF          = 6 ' Left motor forward
symbol PIN_MLB          = 7 ' Left motor backward
symbol PINC_PWM_L       = 1 ' PWM Left
symbol PINC_PWM_R       = 2 ' PWM Right
symbol PINC_LED_LB      = 3 ' LED: Left Blue
symbol PINC_LED_RR      = 4 ' LED: Right Red
symbol PINC_LED_RG      = 5 ' LED: Right Green
symbol PINC_LED_RB      = 6 ' LED: Right Blue

' Constants for input pins
symbol PINC_DIST        = 0          ' SRF05 Echo
symbol PINC_BUMPER      = 7          ' Bumper
symbol PINC_BUMPER_MASK = %10000000  ' Mask for bumper switch

' Servo positions
symbol SERVO_POS_LEFT         = 77
symbol SERVO_POS_LITTLE_LEFT  = 132
symbol SERVO_POS_FORWARD      = 142
symbol SERVO_POS_LITTLE_RIGHT = 152
symbol SERVO_POS_RIGHT        = 207

' Delays and durations
symbol DELAY_BUMP_BACK_UP        = 800
symbol DELAY_BUMP_TURN           = 800
symbol DELAY_SERVO_TURN_FULL     = 500  ' From left to right or from right to left
symbol DELAY_SERVO_TURN_HALF     = 250  ' From center to left or right (or vice versa)
symbol DELAY_FRONT_OBSTACLE_STOP = 800  ' Delay needed for stopping (for sure) when there's an obstacle ahead
symbol DELAY_STOP                = 200  ' Delay needed for stopping
symbol DELAY_TURN                = 400
symbol DELAY_SMALL_TURN          = 200
symbol DELAY_TURN_STOP           = 200  ' Delay needed for stopping after turning
symbol DELAY_SIGNAL              = 200  ' "Signal light" delay
symbol DELAY_BLINK               = 25   ' Eye blink
symbol DELAY_REST_STOP           = 1000 ' Stopping time when resting
symbol DUR_DRIVE_MIN             = 2000 ' Min duration for driving forward
symbol DUR_DRIVE_VAR             = 1000 ' Variance in drive forward duration

' Counts
symbol COUNT_SIGNAL     = 5  ' Signal count
symbol COUNT_MIN_WORK   = 10 ' Must do something before resting

' Obstacle avoidance related constants
symbol INT_CHECK_DIST     = 100 ' Distance checking interval when driving forward
symbol DIST_OBSTACLE      = 70  ' Readings lower than this are considered an obstacle ("SRF05 units" not cm)
symbol DIST_OBSTACLE_SAFE = 100 ' Readings higher than this are considered safe enough to check all distances ("SRF05 units" not cm)
symbol DIST_ENOUGH_ROOM   = 150 ' Readings higher then this are considered to have enough free room to move forward  ("SRF05 units" not cm)

' State masks
symbol STATE_NAVIGATE = %00000010 ' Are we navigating?

' Variables
symbol varb0        = b27 ' A general use variable. Not used to store anything.
symbol varb1        = b26 ' A general use variable. Not used to store anything.
symbol varw0        = w13 ' A general use variable. Not used to store anything.
symbol varw1        = w12 ' A general use variable. Not used to store anything.
symbol dist_left    = w0  ' Distance to left
symbol dist_right   = w1  ' Distance to right
symbol dist_forward = w2  ' Distance to forward
symbol rand         = w3  ' Random
symbol servop       = b8  ' Servo position
symbol command      = b9  ' Command code
symbol player       = b10 ' Player id

' Commands
symbol CMD_FORWARD = 1
symbol CMD_BACKWARD = 2
symbol CMD_LEFT = 3
symbol CMD_RIGHT = 4
symbol CMD_ACTION = 5

' *** Initialization code ***
init:
	servop = SERVO_POS_FORWARD
	servo PIN_SERVO,servop ' initialise servo
	
	' Init random with floating ADC values (4 LSBs per channel)
	readadc10 0, rand : rand = rand << 4
	readadc10 1, varw0 : varw0 = varw0 & $000F : rand = rand | varw0 : rand = rand << 4
	readadc10 2, varw0 : varw0 = varw0 & $000F : rand = rand | varw0 : rand = rand << 4
	readadc10 3, varw0 : varw0 = varw0 & $000F : rand = rand | varw0
	
	sertxd("Hello! I'm receiving now.")
	
' *** Main loop ***
main:
	' Read commands from programming cable
	serrxd varb0
	command = varb0 & %00000111
	player = varb0 & %00111000 >> 3
	select command
		case CMD_FORWARD
			gosub eyes_blue
		case CMD_BACKWARD
			gosub eyes_red
		case CMD_LEFT
			servop = servop - 5
			if servop < SERVO_POS_LEFT then
				servop = SERVO_POS_LEFT
			endif
		case CMD_RIGHT
			servop = servop + 5
			if servop > SERVO_POS_RIGHT then
				servop = SERVO_POS_RIGHT
			endif
		case CMD_ACTION
			gosub beep
			gosub eyes_green
	endselect
	servopos PIN_SERVO, servop
	pause 50
	sertxd("Got command ", #command, " from player ", #player);
	goto main

' *** Beep ***
beep:
	tune PIN_SOUND, 9,($10)
	low PIN_SOUND
	return

' *** Distance measuring ***
read_distance_forward:
	servopos PIN_SERVO, SERVO_POS_FORWARD
	pause DELAY_SERVO_TURN_HALF
	pulsin PINC_DIST, 1, dist_forward
	return
	
' *** Eyes code ***
eyes_off:
	low  PIN_LED_LR        ' LED: Left Red
	low  PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	low  portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return
	
eyes_red:
	high PIN_LED_LR        ' LED: Left Red
	low  PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	high portc PINC_LED_RR ' LED: Right Red
	low  portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return
	
eyes_green:
	low  PIN_LED_LR        ' LED: Left Red
	high PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	high portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return

eyes_blue:
	low  PIN_LED_LR        ' LED: Left Red
	low  PIN_LED_LG        ' LED: Left Green
	high portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	low  portc PINC_LED_RG ' LED: Right Green
	high portc PINC_LED_RB ' LED: Right Blue
	return

eyes_turn_left_red_green:
	low  PIN_LED_LR        ' LED: Left Red
	high PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	high portc PINC_LED_RR ' LED: Right Red
	low  portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return

eyes_turn_right_red_green:
	high PIN_LED_LR        ' LED: Left Red
	low  PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	high portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return
	
eyes_turn_left_green_blue:
	low  PIN_LED_LR        ' LED: Left Red
	high PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	low  portc PINC_LED_RG ' LED: Right Green
	high portc PINC_LED_RB ' LED: Right Blue
	return

eyes_turn_right_green_blue:
	low  PIN_LED_LR        ' LED: Left Red
	low  PIN_LED_LG        ' LED: Left Green
	high portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	high portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return
	
eyes_signal_left_red_green:
	for varb0 = 1 to COUNT_SIGNAL
		gosub eyes_turn_left_red_green
		pause DELAY_SIGNAL
		gosub eyes_red
		pause DELAY_SIGNAL
	next
	return

eyes_signal_right_red_green:
	for varb0 = 1 to COUNT_SIGNAL
		gosub eyes_turn_right_red_green
		pause DELAY_SIGNAL
		gosub eyes_red
		pause DELAY_SIGNAL
	next
	return

eyes_signal_left_green_blue:
	for varb0 = 1 to COUNT_SIGNAL
		gosub eyes_turn_left_green_blue
		pause DELAY_SIGNAL
		gosub eyes_blue
		pause DELAY_SIGNAL
	next
	return

eyes_signal_right_green_blue:
	for varb0 = 1 to COUNT_SIGNAL
		gosub eyes_turn_right_green_blue
		pause DELAY_SIGNAL
		gosub eyes_blue
		pause DELAY_SIGNAL
	next
	return
	
eyes_blink:
	gosub eyes_off
	pause DELAY_BLINK
	gosub eyes_green
	return

eyes_green_left:
	low  PIN_LED_LR        ' LED: Left Red
	high PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	low  portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return

eyes_green_right:
	low  PIN_LED_LR        ' LED: Left Red
	low  PIN_LED_LG        ' LED: Left Green
	low  portc PINC_LED_LB ' LED: Left Blue
	low  portc PINC_LED_RR ' LED: Right Red
	high portc PINC_LED_RG ' LED: Right Green
	low  portc PINC_LED_RB ' LED: Right Blue
	return
