/*
	Remote Soccer Bot Client - Proof of Concept
	
	This application connects to a TCP port on a host. Port and host are specified on GUI.
	There's not anykind of authentication nor encryption implemented so USE THIS AT YOUR OWN RISK.
	
	$Id: RSBClient.java 336 2010-09-09 17:28:20Z nuumio $
*/

import java.io.*;
import java.net.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class RSBClient implements ActionListener, KeyListener{
	public static final String DEFAULT_HOST = "localhost";
	public static final int DEFAULT_PORT = 12345;
	
	public static final int NET_COMMAND_BYTE_FORWARD = 1;
	public static final int NET_COMMAND_BYTE_BACKWARD = 2;
	public static final int NET_COMMAND_BYTE_LEFT = 3;
	public static final int NET_COMMAND_BYTE_RIGHT = 4;
	public static final int NET_COMMAND_BYTE_ACTION = 5;
	
	private JTextField serverAddressField = null;
	private JTextField serverPortField = null;
	
	private OutputStream outputStream = null;

	public static void main(String args[]) {
		new RSBClient();
	}
	
	public RSBClient() {
		// Create and set up the window with simple layout.
		// (Well GridBagLayout isn't exactly the simplest thing...)
		JButton button;
		JLabel label;
		
		JFrame frame = new JFrame("LMR Soccer Robot Controller");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		Container pane = frame.getContentPane();
	
		pane.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();

		label = new JLabel("Server address");
		c.weightx = 0.2;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 0;
		pane.add(label, c);

		serverAddressField = new JTextField(DEFAULT_HOST);
		c.weightx = 0.8;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = GridBagConstraints.REMAINDER;
		pane.add(serverAddressField, c);
		
		label = new JLabel("Server port");
		c.weightx = 0.2;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 1;
		pane.add(label, c);

		serverPortField = new JTextField(String.valueOf(DEFAULT_PORT));
		c.weightx = 0.8;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1;
		c.gridy = 1;
		c.gridwidth = GridBagConstraints.REMAINDER;
		pane.add(serverPortField, c);
		
		button = new JButton("Connect");
		button.addActionListener(this);
		button.addKeyListener(this);
		button.setInputMap(JComponent.WHEN_FOCUSED, null); // No key input for this button (space doesn't equal clicking)
		c.weightx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(0,0,10,0);
		c.gridx = 1;
		c.gridy = 2;
		c.gridwidth = 1;
		pane.add(button, c);

		button = new JButton("Forward");
		button.addActionListener(this);
		button.addKeyListener(this);
		button.setInputMap(JComponent.WHEN_FOCUSED, null); // No key input for this button (space doesn't equal clicking)
		c.weightx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.insets = new Insets(0,0,0,0);
		c.gridx = 1;
		c.gridy = 3;
		c.gridwidth = 1;
		pane.add(button, c);

		button = new JButton("Left");
		button.addActionListener(this);
		button.addKeyListener(this);
		button.setInputMap(JComponent.WHEN_FOCUSED, null); // No key input for this button (space doesn't equal clicking)
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0.5;
		c.gridx = 0;
		c.gridy = 4;
		pane.add(button, c);

		button = new JButton("Backward");
		button.addActionListener(this);
		button.addKeyListener(this);
		button.setInputMap(JComponent.WHEN_FOCUSED, null); // No key input for this button (space doesn't equal clicking)
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0.5;
		c.gridx = 1;
		c.gridy = 4;
		pane.add(button, c);

		button = new JButton("Right");
		button.addActionListener(this);
		button.addKeyListener(this);
		button.setInputMap(JComponent.WHEN_FOCUSED, null); // No key input for this button (space doesn't equal clicking)
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 0.5;
		c.gridwidth = 3;
		c.gridx = 2;
		c.gridy = 4;
		pane.add(button, c);

		button = new JButton("Action");
		button.addActionListener(this);
		button.addKeyListener(this);
		button.setInputMap(JComponent.WHEN_FOCUSED, null); // No key input for this button (space doesn't equal clicking)
		c.fill = GridBagConstraints.HORIZONTAL;
		c.ipady = 40;      //make this component tall
		c.weighty = 1.0;   //request any extra vertical space
		c.insets = new Insets(10,0,10,0);
		c.gridx = 1;       //aligned with button 2
		c.gridy = 5;
		pane.add(button, c);
		
		label = new JLabel();
		c.weightx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.PAGE_END; //bottom of space
		c.ipady = 0;       //reset to default
		c.gridx = 0;
		c.gridy = 6;
		c.gridwidth = 1;
		pane.add(label, c);
		
		//Display the window.
		frame.pack();
		frame.setVisible(true);
	}
	
	public void actionPerformed(ActionEvent e) {
		String command = e.getActionCommand();
		System.out.println("actionPerformed: " + command);
		if(command.equals("Forward")) {
			doForward();
		} else if(command.equals("Backward")) {
			doBackward();
		} else if(command.equals("Left")) {
			doLeft();
		} else if(command.equals("Right")) {
			doRight();
		} else if(command.equals("Action")) {
			doAction();
		} else if(command.equals("Connect")) {
			openServerConnection();
		}
	}
	public void keyPressed(KeyEvent e) {
		System.out.println("keyPressed: "+ KeyEvent.getKeyText(e.getKeyCode()));
		switch(e.getKeyCode()) {
			case KeyEvent.VK_UP:
				doForward();
				break;
			case KeyEvent.VK_DOWN:
				doBackward();
				break;
			case KeyEvent.VK_LEFT:
				doLeft();
				break;
			case KeyEvent.VK_RIGHT:
				doRight();
				break;
			case KeyEvent.VK_SPACE:
				doAction();
				break;
			default:
				break;
		}
	}
	public void keyReleased(KeyEvent e) {
		//System.out.println("keyReleased: "+ KeyEvent.getKeyText(e.getKeyCode()));
	}
	public void keyTyped(KeyEvent e) {
		//System.out.println("keyTyped: "+ KeyEvent.getKeyText(e.getKeyCode()));
	}
	
	private void openServerConnection() {
		String host = this.serverAddressField.getText();
		int port = Integer.parseInt(this.serverPortField.getText());
		try {
			Socket serverConnection = new Socket(host, port);
			this.outputStream = serverConnection.getOutputStream();
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	private void doForward() {
		if(this.outputStream != null) {
			try {
				this.outputStream.write(NET_COMMAND_BYTE_FORWARD);
				System.out.println("doForward");
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
	}
	private void doBackward() {
		if(this.outputStream != null) {
			try {
				this.outputStream.write(NET_COMMAND_BYTE_BACKWARD);
				System.out.println("doBackward");
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
	}
	private void doLeft() {
		if(this.outputStream != null) {
			try {
				this.outputStream.write(NET_COMMAND_BYTE_LEFT);
				System.out.println("doLeft");
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
	}
	private void doRight() {
		if(this.outputStream != null) {
			try {
				this.outputStream.write(NET_COMMAND_BYTE_RIGHT);
				System.out.println("doRight");
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
	}
	private void doAction() {
		if(this.outputStream != null) {
			try {
				this.outputStream.write(NET_COMMAND_BYTE_ACTION);
				System.out.println("doAction");
			} catch(Exception e) {
				e.printStackTrace();
			}
		}
	}
}
