/*
	Remote Soccer Bot Server - Proof of Concept
	
	This application listens to TCP port DEFAULT_PORT and accepts up to MAX_PLAYERS incoming connections.
	There's not anykind of authentication nor encryption implemented so USE THIS AT YOUR OWN RISK.
	
	$Id: RSBServer.java 336 2010-09-09 17:28:20Z nuumio $
*/

import java.io.*;
import java.net.*;
import java.util.*;
import gnu.io.*;

public class RSBServer {
	public static final int DEFAULT_PORT = 12345;
	public static final int MAX_PLAYERS = 6;
	
	public static final int NET_COMMAND_BYTE_FORWARD = 1;
	public static final int NET_COMMAND_BYTE_BACKWARD = 2;
	public static final int NET_COMMAND_BYTE_LEFT = 3;
	public static final int NET_COMMAND_BYTE_RIGHT = 4;
	public static final int NET_COMMAND_BYTE_ACTION = 5;
	
	// Helper class for reading stuff coming from a client
	public class ClientConnection implements Runnable {
		private Socket clientSocket = null;
		private int clientNumber = -1;
		private SerialPort serialPort = null;
		private OutputStream serialOut = null;
		
		public ClientConnection(Socket s, int n, SerialPort serialPort) throws Exception {
			// Store socket and client number
			this.clientSocket = s;
			this.clientNumber = n;
			this.serialPort = serialPort;
			this.serialOut = serialPort.getOutputStream();
		}
		
		public void run() {
			System.out.println("Player " + this.clientNumber + ": Started");
			try {
				InputStream i = this.clientSocket.getInputStream();
				int bytein = 0;
				// Keep reading the inputstream until the end is reached
				while((bytein = i.read()) != -1) {
					// Just dump the value stdout
					System.out.println("Player " + this.clientNumber + ": Got " + bytein + " from client");
					// Format command byte to uC
					// Bits are [7:6] = Not used
					//          [5:3] = Player id
					//          [2:0] = Command id
					int command = this.clientNumber << 3 | bytein;
					synchronized(this.serialPort) {
						System.out.println("Player " + this.clientNumber + ": Sending " + command + " to uC");
						this.serialOut.write(command);
						// Picaxe serrxd wants 13 and 10 too
						this.serialOut.write(13);
						this.serialOut.write(10);
						this.serialOut.flush();
					}
				}
			} catch(Exception e) {
				e.printStackTrace();
			}
			System.out.println("Player " + this.clientNumber + ": Exited");
		}
	}

    // Helper class to read and dump everything we get from uC
	// Copied from http://rxtx.qbang.org/wiki/index.php/Two_way_communcation_with_the_serial_port
    public static class SerialReader implements Runnable 
    {
        InputStream in;
        
        public SerialReader ( InputStream in )
        {
            this.in = in;
        }
        
        public void run ()
        {
            byte[] buffer = new byte[1024];
            int len = -1;
            try
            {
                while ( ( len = this.in.read(buffer)) > -1 )
                {
					if(len > 0) {
						System.out.println("SERVER: uC said: " + new String(buffer,0,len));
					}
                }
            }
            catch ( IOException e )
            {
                e.printStackTrace();
            }            
        }
    }

	
	private ServerSocket serverSocket = null;
	private Vector<ClientConnection> clients = null;
	private int clientNumber = 0;
	private SerialPort serialPort = null;

	/**
	 * Copied from http://rxtx.qbang.org/wiki/index.php/Discovering_available_comm_ports
	 * Modified a little
     * @return    A HashSet containing the CommPortIdentifier for all serial ports that are not currently being used.
     */
    public static Vector<CommPortIdentifier> getAvailableSerialPorts() {
        Vector<CommPortIdentifier> v = new Vector<CommPortIdentifier>();
        Enumeration thePorts = CommPortIdentifier.getPortIdentifiers();
        while (thePorts.hasMoreElements()) {
            CommPortIdentifier com = (CommPortIdentifier) thePorts.nextElement();
            switch (com.getPortType()) {
            case CommPortIdentifier.PORT_SERIAL:
                try {
                    CommPort thePort = com.open("CommUtil", 50);
                    thePort.close();
                    v.add(com);
                } catch (PortInUseException e) {
                    //System.out.println("Port, "  + com.getName() + ", is in use.");
                } catch (Exception e) {
                    System.err.println("Failed to open port " +  com.getName());
                    e.printStackTrace();
                }
            }
        }
        return v;
    }
	
	public static void main(String args[]) {
		Vector<CommPortIdentifier> ports = getAvailableSerialPorts();
		int i;
		System.out.println("\nAvailable serial ports:");
        for(i = 0; i < ports.size(); i++)
        {
            System.out.println((i + 1) + ": " + ports.elementAt(i).getName() );
        }  
		System.out.print("Choose port [1]: ");
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		String portSelection = null;
		try {
			portSelection = br.readLine();
		} catch(Exception e) {
			e.printStackTrace();
			System.exit(0);
		}
		if(portSelection == null || portSelection.equals("")) {
			i = 0;
		} else {
			try {
				i = Integer.parseInt(portSelection) - 1;
			}catch(NumberFormatException e) {
				i = 0;
			}
			if(i < 0 || i >= ports.size()) {
				i = 0;
			}
		}
		System.out.println("Selected port " + ports.elementAt(i).getName());
		
		SerialPort serialPort = null;
		try {
			serialPort = (SerialPort)ports.elementAt(i).open("RSBServer",2000);
			serialPort.setSerialPortParams(4800,SerialPort.DATABITS_8,SerialPort.STOPBITS_1,SerialPort.PARITY_NONE);
		} catch(Exception e) {
			e.printStackTrace();
			System.exit(0);
		}

		RSBServer server = new RSBServer(serialPort);
		server.run();
		System.out.println("SERVER: Exit");
	}
	
	public RSBServer(SerialPort serialPort) {
		try {
			serverSocket = new ServerSocket(DEFAULT_PORT);
			System.out.println("SERVER: Listening to TCP port " + DEFAULT_PORT);
		} catch(Exception e) {
			e.printStackTrace();
		}
		this.serialPort = serialPort;
		try {
			InputStream in = this.serialPort.getInputStream();
			(new Thread(new SerialReader(in))).start();
		} catch(Exception e) {
			e.printStackTrace();
		}
		clients = new Vector<ClientConnection>();
	}
	
	public void run() {
		if(this.serverSocket != null) {
			System.out.println("SERVER: Waiting for connections");
			while(true) {
				try {
					Socket clientSocket = serverSocket.accept();
					System.out.println("SERVER: Connection from " + clientSocket.getRemoteSocketAddress());
					ClientConnection conn = new ClientConnection(clientSocket, this.clientNumber, this.serialPort);
					Thread clientThread = new Thread(conn, "Client-thread-" + this.clientNumber);
					this.clientNumber++;
					this.clients.add(conn);
					clientThread.start();
				} catch(Exception e) {
					e.printStackTrace();
				}
				
				// Break out of while loop if MAX_PLAYERS reached
				if(this.clients.size() >= MAX_PLAYERS) {
					System.out.println("SERVER: Player limit (" + MAX_PLAYERS + ") reached. Exiting.");
					break;
				}
			}
			// We are just rude and exit
			
		} else {
			System.err.println("SERVER: Error! serverSocket is null.");
		}
	}
}
